@@ -97,6 +97,14 @@ class Sapi5SharedEngine(EngineBase, DelegateTimerManagerInterface):
     #-----------------------------------------------------------------------
 
     def __init__(self, retain_dir=None):
+        """
+        :param retain_dir: Retains recognized audio and/or  metadata in the
+          given directory, saving audio to ``retain_[timestamp].wav`` file
+          and metadata to ``retain.tsv``.
+
+          Disabled by default (``None``).
+        :type retain_dir: str|None
+        """
         EngineBase.__init__(self)
         DelegateTimerManagerInterface.__init__(self)
 
@@ -220,7 +228,19 @@ class Sapi5SharedEngine(EngineBase, DelegateTimerManagerInterface):
     # Miscellaneous methods.
 
     def mimic(self, words):
-        """ Mimic a recognition of the given *words*. """
+        """
+        Mimic a recognition of the given *words*.
+
+        .. note:: This method has a few quirks to be aware of:
+
+           #. Mimic can fail to recognize a command if the relevant grammar
+              is not yet active.
+           #. Mimic does not work reliably with the shared recognizer unless
+              there are one or more exclusive grammars active.
+           #. Mimic can **crash the process** in some circumstances, e.g.
+              when mimicking non-ASCII characters.
+
+        """
         self._log.debug("SAPI5 mimic: %r" % (words,))
         if isinstance(words, string_types):
             phrase = words
@@ -304,55 +324,39 @@ class Sapi5SharedEngine(EngineBase, DelegateTimerManagerInterface):
     def _has_quoted_words_support(self):
         return False
 
-    def process_grammars_context(self, window=None):
-        """
-            Enable/disable grammars & rules based on their current contexts.
-
-            This must be done preemptively because WSR doesn't allow doing it
-            upon/after the utterance start has been detected. The engine
-            should call this automatically whenever the foreground application
-            (or its title) changes. But the user may want to call this
-            manually to update when custom contexts.
-
-            The *window* parameter is optional window information, which can
-            be passed in as an optimization if it has already been gathered.
-
-        """
-
-        if window is None: window = Window.get_foreground()
-        for grammar in self.grammars:
-            # Prevent 'notify_begin()' from being called.
-            if grammar.name == "_recobs_grammar":
-                continue
-            grammar.process_begin(window.executable, window.title,
-                                  window.handle)
-
     def _do_recognition(self):
         """
             Recognize speech in a loop.
 
-            This will also call any scheduled timer functions and ensure that
-            the correct window context is used.
+            This will also call any scheduled timer functions and ensure
+            that the correct window context is used.
 
         """
 
         # Register for window change events to activate/deactivate grammars
-        # and rules on window changes. This is done here because the SAPI5
-        # 'OnPhraseStart' grammar callback is called after grammar state
-        # changes are allowed.
+        # and rules on window changes, including window title changes. This
+        # is done here because the SAPI5 'OnPhraseStart' grammar callback is
+        # called after grammar state changes are allowed.
         WinEventProcType = WINFUNCTYPE(None, HANDLE, DWORD, HWND, LONG,
                                        LONG, DWORD, DWORD)
 
         self._last_foreground_window = None
+        self._last_foreground_window_title = None
 
         def callback(hWinEventHook, event, hwnd, idObject, idChild,
                      dwEventThread, dwmsEventTime):
             window = Window.get_foreground()
             # Note: hwnd doesn't always match window.handle, even when
             # foreground window changed (and sometimes it didn't change)
-            if window != self._last_foreground_window:
+            window_changed = (
+                window != self._last_foreground_window or
+                window == self._last_foreground_window and
+                window.title != self._last_foreground_window_title
+            )
+            if window_changed:
                 self.process_grammars_context(window)
                 self._last_foreground_window = window
+                self._last_foreground_window_title = window.title
 
         def set_hook(win_event_proc, event_type):
             return windll.user32.SetWinEventHook(
@@ -432,7 +436,7 @@ class Sapi5InProcEngine(Sapi5SharedEngine):
              - As a *str* containing the description of the audio source
                to use, or a substring thereof
 
-            This class' method *get_audio_sources()* can be used to
+            The :meth:`get_audio_sources()` method can be used to
             retrieve the available sources together with their indices
             and descriptions.
 
@@ -505,7 +509,8 @@ class GrammarWrapper(GrammarWrapperBase):
         self.grammar.process_begin(window.executable, window.title,
                                    window.handle)
 
-    def recognition_callback(self, StreamNumber, StreamPosition, RecognitionType, Result):
+    def recognition_callback(self, StreamNumber, StreamPosition,
+                             RecognitionType, Result):
         try:
             newResult = Dispatch(Result)
             phrase_info = newResult.PhraseInfo
@@ -659,6 +664,7 @@ class GrammarWrapper(GrammarWrapperBase):
         func = getattr(self.grammar, "process_recognition_other", None)
         self._process_grammar_callback(func, words=False, results=None)
 
-    def recognition_failure_callback(self, StreamNumber, StreamPosition, Result):
+    def recognition_failure_callback(self, StreamNumber, StreamPosition,
+                                     Result):
         func = getattr(self.grammar, "process_recognition_failure", None)
         self._process_grammar_callback(func, results=Dispatch(Result))
