@@ -49,6 +49,12 @@ class X11Window(BaseWindow):
         Window control methods such as :meth:`close` will return ``True``
         if successful.
 
+        This class requires the following external programs:
+
+        * ``wmctrl``
+        * ``xdotool``
+        * ``xprop``
+
     """
 
     _log = logging.getLogger("window")
@@ -57,25 +63,10 @@ class X11Window(BaseWindow):
     # Methods and attributes for running commands.
 
     # Commands
+    wmctrl = "wmctrl"
     xdotool = "xdotool"
     xprop = "xprop"
 
-    @classmethod
-    def _run_xdotool_command(cls, arguments):
-        return cls._run_command(cls.xdotool, arguments)
-
-    @classmethod
-    def _run_xdotool_command_simple(cls, arguments):
-        # Run the command and return whether or not it succeeded based on
-        # the return code.
-        stdout, return_code = cls._run_command(cls.xdotool, arguments)
-        if stdout: print(stdout)
-        return return_code == 0
-
-    @classmethod
-    def _run_xprop_command(cls, arguments):
-        return cls._run_command(cls.xprop, arguments)
-
     @classmethod
     def _run_command(cls, command, arguments):
         """
@@ -111,8 +102,9 @@ class X11Window(BaseWindow):
             if isinstance(stderr, binary_type):
                 stderr = stderr.decode(encoding)
 
+            # Print error messages to stderr. Filter BadWindow messages.
             stderr = stderr.rstrip()
-            if stderr:
+            if stderr and "BadWindow" not in stderr:
                 print(stderr, file=sys.stderr)
 
             # Return the process output and return code.
@@ -123,6 +115,30 @@ class X11Window(BaseWindow):
                            full_readable_command, e, command)
             raise e
 
+    @classmethod
+    def _run_command_simple(cls, exe, arguments):
+        # Run the command and return whether or not it succeeded based on
+        # the return code.
+        stdout, return_code = cls._run_command(exe, arguments)
+        if stdout: print(stdout)
+        return return_code == 0
+
+    @classmethod
+    def _run_wmctrl_command_simple(cls, arguments):
+        return cls._run_command_simple(cls.wmctrl, arguments)
+
+    @classmethod
+    def _run_xdotool_command(cls, arguments):
+        return cls._run_command(cls.xdotool, arguments)
+
+    @classmethod
+    def _run_xdotool_command_simple(cls, arguments):
+        return cls._run_command_simple(cls.xdotool, arguments)
+
+    @classmethod
+    def _run_xprop_command(cls, arguments):
+        return cls._run_command(cls.xprop, arguments)
+
     #-----------------------------------------------------------------------
     # Class methods to create new Window objects.
 
@@ -226,7 +242,6 @@ class X11Window(BaseWindow):
         args = ['-id', self.id] + list(properties)
         stdout, return_code = self._run_xprop_command(args)
         if return_code > 0:
-            if stdout: print(stdout)
             return {}
 
         for line in stdout.split('\n'):
@@ -342,12 +357,12 @@ class X11Window(BaseWindow):
     def _get_window_module(self):
         # Get the executable using the process ID and psutil.
         pid = self.pid
-        if pid is None:
+        if pid == -1:
             self._executable = ''
         elif self._executable == -1:
-            for p in psutil.process_iter(attrs=['pid', 'exe']):
+            for p in psutil.process_iter(attrs=['pid', 'exe', 'name']):
                 if p.info['pid'] == pid:
-                    self._executable = p.info['exe']
+                    self._executable = p.info['exe'] or p.info['name']
                     return self._executable
 
             # Set to '' if it wasn't found.
@@ -437,17 +452,19 @@ class X11Window(BaseWindow):
         return self._run_xdotool_command_simple(['windowminimize',
                                                  self.id])
 
-    def _toggle_maximize(self):
-        # Doesn't seem possible with xdotool. We'll try pressing a-f10
-        # with the window focused. Only maximize if set_foreground()
-        # succeeded.
-        return self.set_foreground() and self._run_xdotool_command_simple([
-            'keydown', 'Alt_L', 'key', 'F10', 'keyup', 'Alt_L'
+    def _toggle_maximize(self, is_maximized):
+        # Use wmctrl to add or remove the maximized window properties from
+        # the window's _NET_WM_STATE set.
+        # Note: this should be possible with xprop, but is not supported.
+        maximized_props = 'maximized_vert,maximized_horz'
+        add_remove = 'remove' if is_maximized else 'add'
+        return self._run_wmctrl_command_simple([
+            '-ir', self.id, '-b', '%s,%s' % (add_remove, maximized_props)
         ])
 
     def maximize(self):
         if not self.is_maximized:
-            return self._toggle_maximize()
+            return self._toggle_maximize(False)
         return True  # already maximized.
 
     def restore(self):
@@ -457,7 +474,7 @@ class X11Window(BaseWindow):
                 'windowactivate', self.id
             ])
         elif self._is_maximized(state):
-            return self._toggle_maximize()
+            return self._toggle_maximize(True)
         else:
             # True if already restored or False if no _NET_WM_STATE.
             return state is not None
